/*
 * Ovaj fajl sadrzi implementacije funkcija
 * koje su deklarisane u protoco.h fajlu
 * 
 * Imajuci u vidu zamisljen protokol ove
 * funkcije ga implementiraju na jedan od
 * mogucih nacina
 * 
 * Generalno posmatrajuci, nacin rada je sledeci:
 * 1. Kada saljemo odredjeni niz karaktera,
 * broj karaktera se belezi u zaglavlju, zatim
 * se poruka 'zalepi' za zaglavlje i doda joj se
 * \0 karakter. On ne ulazi u broj karaktera u
 * zaglavlju. Zato se salje bafer koji ima duzinu
 * HEADERLEN + messagelen + 1. Ovako se celokupan
 * 'paket' posmatra kao string.
 * 
 * 2. Prihvatanje 'paketa' se obavlja jednostavnije.
 * Procita se prvo HEADERLEN bajtova iz sistemskog bafera
 * i tako se odredi koliko jos bajtova treba da se
 * ucita, kako bi se formirao validan 'paket'. Pored
 * broja karaktera, vodi se racuna i o \0 koji se nalazi
 * iza njih, tako da se sa sistemskog soketa povlaci
 * messagelen+1 bajt.
 * 
 * Autor: Ikac, ikac.ikax@gmail.com
 */
 
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "protocol.h"

int sendstring(int sockfd, char * buffer)
{
  int n, len;
  char format[4];
  char headerbuffer[HEADERLEN];
  
  len = strlen(buffer); //odredjujemo duzinu bafera
  
  sprintf(format, "%%0%dd", HEADERLEN);//pravimo format %04d
  
  sprintf(headerbuffer, format, len);//formiranje zaglavlja 
  
  /*
   * kopira se sadrzaj hedera a zatim i sadrzaj bafera
   * u bafer koji ce nam sluziti za slanje
   */
  char buffer_out[HEADERLEN+len+1];
  strcpy(buffer_out, headerbuffer);
  strcpy(buffer_out+ HEADERLEN, buffer);
  
  /*
   * stavljamo nul karakter da bi
   * se sekvenca tretirala kao string
   */
  buffer_out[HEADERLEN+len] = '\0';
    
  n = write(sockfd, buffer_out, HEADERLEN+len+1);
  
  #ifdef _DEBUG_
    perror("Sending message");
  #endif
  
  return n;
}

int receivestring(int sockfd, char ** buffer)
{
  int n;
  char headerbuffer[HEADERLEN];  
  
  n = read(sockfd, headerbuffer, HEADERLEN);//citanje 4 bajta za heder
  int messagelen = atoi(headerbuffer);//pretvaranje hedera u broj koji oznacava duzinu poruke u bajtovima
    
  *buffer = malloc(sizeof(char)*(messagelen+1));//+1 bajt za nul karakter
  memset(*buffer, ' ', messagelen+1);//nulovanje bafera
  n = read(sockfd, *buffer, messagelen+1);//uzimanje iz sistemskog bafera ukljucujuci i nevidljivi nul karakter (+1)
  
  #ifdef _DEBUG_
    perror("Receiving message");  
  #endif
  
  if(n < 0) return n;
  else return messagelen;
}

